//
//    File: mview.h
//
//    (C) 2000 Helmut Cantzler
//
//    Licensed under the terms of the Lesser General Public License.
//

#ifndef _MVIEW_H
#define _MVIEW_H

#include <qmainwindow.h>
#include <qtoolbutton.h>
#include <qmap.h>
#include <mesh.h>

#include "glmesh.h"

#define SWITCHES 15

#define MVIEW_TEXTURE 0
#define MVIEW_SOLID 1
#define MVIEW_WIRE 2
#define MVIEW_FRONTLINES 3
#define MVIEW_POINTS 4
#define MVIEW_FEATURES 5
#define MVIEW_PICK 6
#define MVIEW_TEXT 7
#define MVIEW_LIGHTING 8
#define MVIEW_SHAPE 9
#define MVIEW_NORMALS 10
#define MVIEW_CUT 11
#define MVIEW_FILTER 12
#define MVIEW_FEATURE 13
#define MVIEW_RATIO 14

class QMultiLineEdit;
class QToolBar;
class QPopupMenu;
class QSlider;
class QLabel;

class MView: public QMainWindow
{
  Q_OBJECT
 public:
    MView(int argc, char **argv);
    ~MView();
    
 public slots:
    void togglePick();
    void toggleTextureMapping();

    void setTexture();

 private slots:
    void load_mesh();
    void load_feature();
    void save_as(int item);
    void takeShot(int item);

    void removeDoublePoints();
    void negateNormals();

    void setSolid();
    void setWire();
    void setFrontlines();
    void setPoints();
    void setFeatures();
    void toggleLighting();
    void toggleShapeColors();
    void toggleSurfaceNormals();
    void toggleCutBackFaces();
    void toggleBilinearFiltering();
    void toggleFeatures();
    void toggleAspectRatio();
    void setBackgroundColor();

    void reset_view();
    void set_view();
    void save_view();
    void clear_views();
    void load_view(int id);

    void about();
    void aboutQt();

 protected:
    void timerEvent(QTimerEvent *e);

 private:
    void set_status();
    void set_glmesh_defaults();
    void set_mview_defaults();
    int load_mesh_file(const char *fileName, Mesh **mesh);

    GLMesh *glmesh;
    GLMesh_Settings *settings;

    QString name;
    QToolButton *pick, *text;
    QSlider *clipping;
    QPopupMenu *save, *screenshot, *modes, *viewpoint;
    QMap<int, GLMesh_Settings> viewpoints;

    int *ids, displayMode;
};

#endif
