//
//    File: texture_image.h
//
//    (C) 2000 Helmut Cantzler
//
//    Licensed under the terms of the Lesser General Public License.
//

#ifndef _TEXTURE_IMAGE_H
#define _TEXTURE_IMAGE_H

#include <qmainwindow.h>
#include <qimage.h>
#include <qpixmap.h>

#include <gandalf/vision/homog33_fit.h>

#include "mesh.h"
#include "glmesh.h"

class MView;

typedef struct
{
  unsigned int x, y;
} Point;

class TextureImage : public QWidget
{
  Q_OBJECT
 public:
  TextureImage(QWidget *parent);

  const unsigned int* getHomoPoints(void);
  const char* getFileName(void) const;

  int width(void) const { return img.width(); }
  int height(void) const { return img.height(); }

 public slots:
  void loadImage(void);
  void clearImage(void);

 protected:
  void mousePressEvent (QMouseEvent *e);
  void paintEvent(QPaintEvent *e);

 private:
  void drawPoint(Point p, unsigned int color);

  QString fn, fn_short;
  QImage img;
  QPixmap pix;

  int homo_nr;
  Point homo_points[4];
};

class TextureImageWindow : public QMainWindow
{
  Q_OBJECT
 public:
  TextureImageWindow(MView *parent, GLMesh *glmesh, Mesh *mesh, const char *name=0);

 private slots:
  void clearSelection(void);
  int texture_mapping(void);
  void calc_projection(float *v, Gan_Matrix33 *h, float *x, float *y);

 private:
  TextureImage *img;
  MView *parent;
  GLMesh *glmesh;
  Mesh *mesh;
};

#endif
